;;;
;;; raven-variables.lisp
;;;
;;; Assorted variables.
;;;

(in-package :nevermore)


;; Internal memory spaces

(defvar *microstack* (make-array '(64) :element-type '(unsigned-byte 32)
				:initial-element 0)
  "Micro-stack data storage.")

(defvar *pdl-buffer* (make-array '(#x400) :element-type '(unsigned-byte 32)
				 :initial-element 0))

(defvar *level-1-map* (make-array '(#x1000) :element-type '(unsigned-byte 32)
				  :initial-element 0)
  "Level-1 Memory Map storage")

(defvar *level-2-control* (make-array '(#x1000)
				      :element-type '(unsigned-byte 32)
				      :initial-element 0)
  "Level-2 Memory Map Control storage")

(defvar *level-2-address* (make-array '(#x1000)
				      :element-type '(unsigned-byte 32)
				      :initial-element 0)
  "Level-2 Memory Map Address storage")

(defvar *d-memory* (make-array '(#x1000) :element-type '(unsigned-byte 32)
			       :initial-element 0)
  "Dispatch memory storage")


;; Internal registers

(defvar *microstack-pointer* 0 "Micro-stack pointer")
(defvar *memory-data* 0)
(defvar *virtual-memory-address* 0)
(defvar *q-register* 0 "The Q register.")
(defvar *pdl-buffer-pointer* 0)
(defvar *pdl-buffer-index* 0)
(defvar *location-counter* 0)
(defvar *macroinstruction-buffer* 0 "Here come the Men In Black!")
(defvar *dispatch-constant* 0 "I-Arg?")
(defvar *machine-control-register* 0)


;; Microengine state

(defvar *micro-instruction-pointer* 0
  "Index of next microinstruction to read")

(defvar *last-micro-instruction-pointer* 0
  "Index of last microinstruction read")

(defvar *inhibit-micro-execution* nil
  "Flag for if the next microinstruction should be ignored.")

(defvar *next-micro-instruction* 0
  "Next microinstruction to execute")

(defvar *micro-instruction-trace* t
  "Flag to enable/disable disassembly of microinstructions during step")


;; Condition bits

(defvar *page-fault* nil "T if last memory/map access caused a page fault.")
(defvar *memory-busy* 0 "Number of cycles for memory-busy or 0.")
(defvar *interrupt-pending* nil "T if interrupts are unmasked and an interrupt occurs")
(defvar *nubus-error* nil "T if last NuBus access caused an error")
(defvar *loop-selftest* nil "T if boot prom should loop selftest")

;; NuBus slot assignments

(defvar *cpu-nubus-slot* 6 "NuBus slot the CPU is in (usually 6)")
(defvar *sib-nubus-slot* 5 "NuBus slot the SIB is in (usually 5)")
(defvar *mem-nubus-slot* 4 "NuBus slot the memory is in (usually 4)")
(defvar *nupi-nubus-slot* 2 "NuBus slot the NuPI is in (usually 2)")
; Ethernet in slot 1 when we add it.

;;; EOF
